#include "ndewidgetmaterialcircularprogress.h"
#include <QPropertyAnimation>
#include <QParallelAnimationGroup>
#include <QPainter>
#include <QPainterPath>

namespace NdeWidget {


QMaterialCircularProgress::QMaterialCircularProgress(QWidget *parent)
    : QProgressBar(parent)
{
    init();
}


void QMaterialCircularProgress::init()
{
    m_progressType   = Material::IndeterminateProgress;
    m_penWidth       = 6.25;
    m_size           = 64;
    m_duration       = 2000;

    this->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding));
    group = new QParallelAnimationGroup(this);
    group->setLoopCount(-1);
    refreshAnimation();
}

QMaterialCircularProgress::~QMaterialCircularProgress()
{
}

void QMaterialCircularProgress::refreshAnimation()
{
    group->clear();

    QPropertyAnimation *animation;

    animation = new QPropertyAnimation(this);
    animation->setPropertyName("dashLength");
    animation->setTargetObject(this);
    animation->setEasingCurve(QEasingCurve::InOutQuad);
    animation->setStartValue(0.1);
    animation->setKeyValueAt(0.15, 0.2);
    animation->setKeyValueAt(0.6, 20);
    animation->setKeyValueAt(0.7, 20);
    animation->setEndValue(20);
    animation->setDuration(m_duration);

    group->addAnimation(animation);

    animation = new QPropertyAnimation(this);
    animation->setPropertyName("dashOffset");
    animation->setTargetObject(this);
    animation->setEasingCurve(QEasingCurve::InOutSine);
    animation->setStartValue(0);
    animation->setKeyValueAt(0.15, 0);
    animation->setKeyValueAt(0.6, -7);
    animation->setKeyValueAt(0.7, -7);
    animation->setEndValue(-25);
    animation->setDuration(m_duration);

    group->addAnimation(animation);

    animation = new QPropertyAnimation(this);
    animation->setPropertyName("angle");
    animation->setTargetObject(this);
    animation->setStartValue(0);
    animation->setEndValue(719);
    animation->setDuration(m_duration);

    group->addAnimation(animation);

    group->start();
}

void QMaterialCircularProgress::setProgressType(Material::ProgressType type)
{
    m_progressType = type;
    update();
}

Material::ProgressType QMaterialCircularProgress::getProgressType() const
{
    return m_progressType;
}

void QMaterialCircularProgress::setLineWidth(qreal width)
{
    m_penWidth = width;
    update();
    updateGeometry();
}

qreal QMaterialCircularProgress::getLineWidth() const
{
    return m_penWidth;
}

void QMaterialCircularProgress::setSize(int size)
{
    m_size = size;
    update();
    updateGeometry();
}

int QMaterialCircularProgress::getSize() const
{
    return m_size;
}

void QMaterialCircularProgress::setDuration(int duration)
{
    m_duration = duration;
    refreshAnimation();
    update();
}

qreal QMaterialCircularProgress::getDuration() const
{
    return m_duration;
}


QSize QMaterialCircularProgress::sizeHint() const
{
    const qreal s = m_size + m_penWidth+8;
    return QSize(s, s);
}


void QMaterialCircularProgress::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)


    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    if (!isEnabled())
    {
        QPen pen;
        pen.setCapStyle(Qt::RoundCap);
        pen.setWidthF(m_penWidth);
        pen.setColor(QColor("#222222"));
        painter.setPen(pen);
        painter.drawLine(rect().center()-QPointF(20, 20), rect().center()+QPointF(20, 20));
        painter.drawLine(rect().center()+QPointF(20, -20), rect().center()-QPointF(20, -20));
        return;
    }

    if (Material::IndeterminateProgress == m_progressType)
    {
        painter.translate(width()/2, height()/2);
        painter.rotate(m_angle);
    }

    QPen pen;
    pen.setCapStyle(Qt::RoundCap);
    pen.setWidthF(m_penWidth);
    pen.setColor(QColor("#2c9dfc"));

    if (Material::IndeterminateProgress == m_progressType)
    {
        QVector<qreal> pattern;
        pattern << dashLength()*m_size/50 << 30*m_size/50;

        pen.setDashOffset(dashOffset()*m_size/50);
        pen.setDashPattern(pattern);

        painter.setPen(pen);

        painter.drawEllipse(QPoint(0, 0), m_size/2, m_size/2);
    }
    else
    {
        painter.setPen(pen);

        const qreal x = (width()-m_size)/2;
        const qreal y = (height()-m_size)/2;

        const qreal a = 360*(value()-minimum())/(maximum()-minimum());

        QPainterPath path;
        path.arcMoveTo(x, y, m_size, m_size, 0);
        path.arcTo(x, y, m_size, m_size, 0, a);

        painter.drawPath(path);
    }
}


}
